@@ -1,11 +1,11 @@
 package Zoidberg;
 
-our $VERSION = '0.42';
+our $VERSION = '0.50';
 our $LONG_VERSION =
 "Zoidberg $VERSION
 
-Copyright (c) 2002 Jaap G Karssenberg. All rights reserved. 
-This program is free software; you can redistribute it and/or 
+Copyright (c) 2002 Jaap G Karssenberg. All rights reserved.
+This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl.
 
 This program is distributed in the hope that it will be useful,
@@ -19,39 +19,40 @@ use vars qw/$AUTOLOAD/;
 
 require Cwd;
 require File::Glob;
-require Zoidberg::Config;
 require Zoidberg::Contractor;
 require Zoidberg::Shell;
 require Zoidberg::PluginHash;
 require Zoidberg::StringParser;
 
-use Data::Dumper;
 use Zoidberg::DispatchTable _prefix => '_', 'stack';
-use Zoidberg::Utils 
-	qw/:error :output :fs :fs_engine read_data_file merge_hash is_exec_in_path/;
+use Zoidberg::Utils
+	qw/:error :output :fs :fs_engine read_data_file merge_hash/;
 #use Zoidberg::IPC;
 
 our @ISA = qw/Zoidberg::Contractor Zoidberg::Shell/;
 
-our %Objects; # used to store refs to ALL Zoidberg objects
-our @core_objects = qw/Buffer Intel Prompt Commands History/; # DEPRECATED
+our %OBJECTS; # used to store refs to ALL Zoidberg objects in a process
+our $CURRENT; # current Zoidberg object
+
+our $_base_dir; # relative path for some settings
+our %_settings; ##Insert defaults here##
 
 sub new {
 	my ($class, $self) = @_;
 	$self ||= {};
-	$self->{$_} ||= {} for qw/settings commands aliases events objects vars/;
-	$self->{round_up}++;
-	$self->{topic} ||= '';
+	$$self{$_} ||= {} for qw/settings commands aliases events objects vars/;
+	$$self{no_words} ||= ['PERL']; # parser HACK might be fixed some day
+	$$self{round_up}++;
+	$$self{topic} ||= '';
 
 	bless($self, $class);
 
-	$Objects{"$self"} = $self;
-	$ENV{ZOIDREF} = $self unless ref $ENV{ZOIDREF};
+	$OBJECTS{"$self"} = $self;
+	$CURRENT = $self unless ref( $CURRENT ) eq $class; # could be autovivicated
 	$self->{shell} = $self; # for Contractor
 
 	## settings
-	my %default = %Zoidberg::Config::settings;
-	$self->{settings}{$_} ||= $default{$_} for keys %default;
+	$self->{settings}{$_} ||= $_settings{$_} for keys %_settings;
 	$self->{settings}{$_} || error "You should at least set a config value for $_"
 		for qw/data_dirs cache_dir/;
 	$self->{settings} = merge_hash( read_data_file('settings'), $self->{settings} );
@@ -65,24 +66,27 @@ sub new {
 	tie %commands, 'Zoidberg::DispatchTable', $self, {
 		reload	=> '->reload',
 		exit	=> '->exit',
-		( %{$self->{commands}} )
+		( %{$$self{commands}} )
 	};
-	$self->{commands} = \%commands;
+	$$self{commands} = \%commands;
 
 	## events
 	my %events;
-	tie %events, 'Zoidberg::DispatchTable', $self, $self->{events};
-	$self->{events} = \%events;
-	$self->{events}{postcmd} = sub {
+	tie %events, 'Zoidberg::DispatchTable', $self, $$self{events};
+	$$self{events} = \%events;
+	$$self{events}{envupdate} = sub {
 		my $pwd = Cwd::cwd();
-		@ENV{qw/OLDPWD PWD/} = ($ENV{PWD}, $pwd)
-			unless $pwd eq $ENV{PWD} ;
+		return if $pwd eq $ENV{PWD};
+		@ENV{qw/OLDPWD PWD/} = ($ENV{PWD}, $pwd);
+		$self->broadcast('newpwd');
 	};
+	$$self{events}{readline} = "->_stdin('zoid-$VERSION\$ ')";
+	$$self{events}{readmore} = "->_stdin('> ')";
 
 	## contexts
 	my %contexts;
-	tie %contexts, 'Zoidberg::DispatchTable', $self;
-	$self->{contexts} = \%contexts;
+	tie %contexts, 'Zoidberg::DispatchTable', $self, $$self{contexts};
+	$$self{contexts} = \%contexts;
 
 #    $self->{ipc} = Zoidberg::IPC->new($self);
 #    $self->{ipc}->init;
@@ -104,12 +108,11 @@ sub new {
 
 	## path cache
 	my $file_cache = "$cache_dir/zoid_path_cache" ;
-	if (-s $file_cache) { f_read_cache($file_cache) }
-	else {
-		message 'Initializing PATH cache.';
-		f_index_path();
-	}
-	$self->{events}{precmd} = \&f_wipe_cache;
+	f_read_cache($file_cache) if -s $file_cache;
+	$self->{events}{prompt} = \&f_wipe_cache;
+
+	## let's load the rcfiles
+	$self->source(grep {-f $_} @{$$self{settings}{rcfiles}});
 
 	return $self;
 }
@@ -120,26 +123,25 @@ sub import { bug "You should use Zoidberg::Shell to import from" if @_ > 1 }
 # Main routine #
 # ############ #
 
-sub main_loop { # FIXME use @input_methods instead of buffer
+sub main_loop {
 	my $self = shift;
-	
-	error "Could not initialize plugin 'Buffer'" unless $self->{objects}{Buffer};
-	$self->{_continue}++;
-	
-	while ($self->{_continue}) {
-		$self->broadcast('precmd');
 
-		my $cmd = eval { $self->Buffer->get_string };
-		last unless $self->{_continue}; # buffer can call exit
+	$$self{_continue} = 1;
+	while ($$self{_continue}) {
+		$self->reap_jobs() if @{$$self{jobs}};
+		$self->broadcast('prompt');
+		my $cmd = eval { $$self{events}{readline}->() };
 		if ($@) {
-			complain "\nBuffer died. (You can interrupt zoid NOW)\n$@";
+			complain "\nInput routine died. (You can interrupt zoid NOW)\n$@";
 			local $SIG{INT} = 'DEFAULT';
 			sleep 1; # infinite loop protection
 		}
-		else { 
+		else {
+			$self->reap_jobs() if @{$$self{jobs}};
+			$self->exit() unless defined $cmd || $$self{settings}{ignoreeof};
+			last unless $$self{_continue};
 			$self->broadcast('cmd', $cmd);
-			print STDERR $cmd if $self->{settings}{verbose}; # posix spec
-
+			print STDERR $cmd if $$self{settings}{verbose}; # posix spec
 			$self->shell_string($cmd);
 		}
 	}
@@ -155,21 +157,14 @@ sub print { bug 'deprecated method, use Zoidberg::Utils qw/:output/' }
 
 sub list_objects { [sort keys %{$_[0]{objects}}] }
 
-sub test_object {
-	my ($self, $zoidname, $class) = @_;
-	return 1 if exists $self->{objects}{$zoidname};
-	# FIXME also check class
-	return 0;
-}
-
-sub list_clothes { # includes $self->{vars}
+sub list_clothes {
 	my $self = shift;
 	my @return = map {'{'.$_.'}'} sort @{$self->{settings}{clothes}{keys}};
 	push @return, sort @{$self->{settings}{clothes}{subs}};
 	return [@return];
 }
 
-sub list_vars { return [map {'{'.$_.'}'} sort keys %{$_[0]->{vars}}]; }
+sub list_vars { return [map {'{'.$_.'}'} sort keys %{$_[0]->{vars}}] }
 
 # ############ #
 # Parser stuff #
@@ -181,7 +176,7 @@ sub shell_string {
 			unshift @list, $meta;
 			undef $meta;
 	}
-	local $ENV{ZOIDREF} = $self;
+	local $CURRENT = $self;
 	@list = $$self{stringparser}->split('script_gram', @list);
 	my $e = $$self{stringparser}->error;
 	return complain $e if $e;
@@ -192,108 +187,120 @@ sub shell_string {
 
 sub parse_block  { 
 	# call as late as possible before execution
-	# bit is "broken bit" (also known as "Intel bit")
+	# bit is "pretend bit" (also known as "Intel bit")
+	# $queue is unshift only, no shift - else you can fuck up logic list
 	my ($self, $block, $queue, $bit) = @_;
-	my ($meta, @words) = ({broken => $bit});
+	my ($meta, @words) = ({pretend => $bit});
 
 	# decipher block
 	my $t = ref $block;
-	if (!$t or $t eq 'SCALAR') {
-		$$meta{string} = $t ? $$block : $block;
-		@words = grep {length $_} $self->{stringparser}->split('word_gram', $$meta{string});
-	}
+	if (!$t or $t eq 'SCALAR') { $$meta{string} = $t ? $$block : $block }
 	elsif ($t eq 'ARRAY') {
-		$meta = { %$meta, %{shift @$block} }
-			if ref($$block[0]) eq 'HASH';
+		$meta = { %$meta, %{shift @$block} } if ref($$block[0]) eq 'HASH';
 		@words = @$block;
 	}
 	else { bug "parse tree contains $t reference" }
 
-	return undef unless grep {length $_} @words;
+	# check aliase and other meta stuff
+	my @blocks = $self->parse_macros($meta, @words);
+	if (@blocks > 1) { # alias contained pipe or logic operator
+		bug 'No queue argument given' unless ref $queue;
+		($meta, @words) = @{ shift(@blocks) };
+		unshift @$queue, @blocks;
+	}
+	elsif (@blocks) { ($meta, @words) = @{ shift(@blocks) } }
+	else { return undef }
 
-	($meta, @words) = $self->_filter($meta, @words);
-	return [$meta, @words] if $$meta{context};
+	# check custom filters
+	for my $sub (_stack($$self{contexts}, 'filter')) {
+		my $r = $sub->([$meta, @words]);
+		($meta, @words) = @$r if $r; # skip on undef
+	}
 
 	# check builtin contexts
-	unless ($$meta{context} || $$self{settings}{_no_hardcoded_context}) {
+	unless ($$meta{context} or $$self{settings}{_no_hardcoded_context}) {
 		debug 'trying builtin contexts';
 		my $perl_regexp = join '|', @{$self->{settings}{perl_keywords}};
 		if (
-			(@words == 1) && $words[0] =~ s/^\s*(\w*){(.*)}(\w*)\s*$/$2/s
-			or $$meta{broken} && $words[0] =~ s/^\s*(\w*){(.*)$/$2/s
-		) {
-			if (! $1 or uc($1) eq 'ZOID') {
-				@$meta{qw/context dezoidify _no_words opts/} = ('PERL', 1, 1, $3 || '');
-			}
-			elsif (grep {$_ eq $1} qw/s tr y/) {
-				@$meta{qw/context _no_words/} = ('PERL', 1);
-				$words[0] = $1.'{'.$2.'}'.$3 ;
+			$$meta{string} =~ s/^\s*(\w*){(.*)}(\w*)\s*$/$2/s
+			or $$meta{pretend} and $$meta{string} =~ s/^\s*(\w*){(.*)$/$2/s
+		) { # all kinds of blocks with { ... }
+			unless (length $1) { @$meta{qw/context opts/} = ('PERL', $3 || '') }
+			elsif (grep {$_ eq $1} qw/s m tr y/) {
+				$$meta{string} = $1.'{'.$$meta{string}.'}'.$3; # always one exception
+				@$meta{qw/context opts/} = ('PERL', ($1 eq 'm') ? 'g' : 'p')
 			}
 			else {
 				@$meta{qw/context opts/} = (uc($1), $3 || '');
-				if (
-					$$meta{context} eq 'SH' or $$meta{context} eq 'CMD'
-					or exists $self->{contexts}{$$meta{context}}{word_list}
-				) { # split words agan
-					@words = $self->_split_words($words[0], $$meta{broken});
-				}
+				@words = grep {length $_} $self->{stringparser}->split('word_gram', $$meta{string});
 			}
 		}
-		elsif ($words[0] =~ /^\s*(->|[\$\@\%\&\*\xA3]\S|\w+[\(\{]|($perl_regexp)\b)/s) {
-			@$meta{qw/context dezoidify/} = ('PERL', 1);
-			($meta, @words) = @{ $self->_no_words([$meta, @words]) };
-		} 
+		elsif ($$meta{string} =~ s/^\s*(\w+):\s+//) { # little bit o psh2 compat
+			$$meta{context} = uc $1;
+			shift @words;
+		}
+		elsif ($words[0] =~ /^\s*(->|[\$\@\%\&\*\xA3]\S|\w+::|\w+[\(\{]|($perl_regexp)\b)/s) {
+			$$meta{context} = 'PERL';
+		}
 	}
 
+	return [$meta, @words] if $$meta{pretend} and @words == 1;
+
 	# check custom contexts
 	unless ($$meta{context}) {
 		debug 'trying custom contexts';
-		for (keys %{$self->{contexts}}) {
-			next unless exists $self->{contexts}{$_}{word_list};
-			my $r = $self->{contexts}{$_}{word_list}->([$meta, @words]);
+		for my $pair (_stack($$self{contexts}, 'word_list', 'TAGS')) {
+			my $r = $$pair[0]->([$meta, @words]);
 			unless ($r) { next }
 			elsif (ref $r) { ($meta , @words) = @$r }
-			else { $$meta{context} = length($r) > 1 ? $r : $_ }
+			else { $$meta{context} = length($r) > 1 ? $r : $$pair[1] }
 			last if $$meta{context};
 		}
 	}
-	
-	# check more builtin contexts
-	unless ($$meta{context} || $$self{settings}{_no_hardcoded_context}) {
-		debug 'trying more builtin contexts';
-#		no strict 'refs';
-		if (
-#			defined *{"Zoidberg::Eval::$words[0]"}{CODE} or
-			exists $self->{commands}{$words[0]}
-		) { $$meta{context} = 'CMD' }
-		elsif (
-			($words[0] =~ m!/! and -x $words[0] and ! -d $words[0])
-			or is_exec_in_path($words[0])
-		) { $$meta{context} = 'SH' }
-		$$meta{_is_checked}++ if $$meta{context};
 
-		# hardcoded default context
-		unless ($$meta{context} || $$meta{broken}) {
-			debug 'going for the default context';
-			$$meta{context} = 'SH';
-		} # just leave blank for broken syntax
+	# check default builtin contexts
+	unless ($$meta{context} || $$self{settings}{_no_hardcoded_context}) {
+		debug 'using default contexts';
+		$$meta{context} = exists( $$self{commands}{$words[0]} ) ? 'CMD' : 'SH' ;
 	}
 
-	($meta, @words) = @{ $self->_do_words([$meta, @words]) }
-		if @words && ! @$meta{qw/broken _no_words/} ;
+	if (exists $$self{contexts}{$$meta{context}}{parser}) { # custom parser
+		($meta, @words) = @{ $$self{contexts}{$$meta{context}}{parser}->([$meta, @words]) };
+	}
+	elsif (grep {$$meta{context} eq $_} @{$$self{no_words}}) { # no words
+		if ($$meta{pretend}) { @words = grep {length $_} $self->{stringparser}->split('word_gram', $$meta{string}) }
+		else { @words = ($$meta{string}) }
+	}
+	elsif (@words and ! $$meta{pretend}) { # expand and set topic
+		($meta, @words) = @{ $self->parse_words([$meta, @words]) }; # uses old topic
+		$$self{topic} =
+			exists($$meta{fd}{0}) ? $$meta{fd}{0}[0] :
+			(@words > 1 and $words[-1] !~ /^-/) ? $words[-1] : $$self{topic};
+		$$meta{fork_job} = 1 if $$meta{context} eq 'SH'; # custom contexts do this in parser sub
+	}
 	return [$meta, @words];
 }
 
-sub _filter {
+sub parse_macros {
 	my ($self, $meta, @words) = @_;
 
+	unless (@words) {
+		@words = grep {length $_} $self->{stringparser}->split('word_gram', $$meta{string})
+	}
+	else { delete $$meta{string} } # just to make sure
+
 	# parse environment
 	unless ( $$self{settings}{_no_env} ) {
-		while ($words[0] =~ /^([A-Z][A-Z_\-\d]*)=(.*)/) {
-			$$meta{start} ||= [];
-			my (undef, @w) = @{ $self->_do_words([{_no_topic => 1}, $2]) };
-			$$meta{env}{$1} = join ':', @w;
+		while ($words[0] =~ /^(\w[\w\-]*)=(.*)/) {
 			push @{$$meta{start}}, shift @words;
+			$$meta{start} ||= [];
+			my (undef, @w) = @{ $self->parse_words([{}, $2]) };
+			$$meta{env}{$1} = join ':', @w;
+		}
+		if (! @words and $$meta{env}) { # special case
+			@words = ('export', map $_.'='.$$meta{env}{$_}, keys %{$$meta{env}});
+			$$meta{string} = '';
+			# FIXME FIXME more systematic way to do this
 		}
 	}
 
@@ -308,24 +315,30 @@ sub _filter {
 		# FIXME are it always _2_ words ?
 		$$meta{end} = [ splice @words, -2 ];
 		my $num = $1 || ( ($2 eq '<') ? 0 : 1 );
-		my (undef, @w) = @{ $self->_do_words([{_no_topic => 1}, $$meta{end}[-1]]) };
+		my (undef, @w) = @{ $self->parse_words([{}, $$meta{end}[-1]], 'NO_TOPIC') };
 		my $file = (@w == 1) ? $w[0] : $$meta{end}[-1];
 		$$meta{fd}{$num} = [ $file, $2 ];
 	}
 
-	# check custom filters
-	for (keys %{$self->{contexts}}) {
-		next unless exists $self->{contexts}{$_}{filter};
-		my $r = $self->{contexts}{$_}{filter}->([$meta, @words]);
-		($meta, @words) = @$r if $r; # skip on undef
+	@words = $self->_do_aliases(@words) if @words && ! $$meta{pretend};
+
+	if ($$meta{string}) {
+		if (exists $$meta{end}) {
+			$$meta{string} =~ s/\s*\Q$_\E\s*$//
+				for reverse @{$$meta{end}};
+		}
+		if (exists $$meta{start}) {
+			$$meta{string} =~ s/^\s*\Q$_\E\s*//
+				for @{$$meta{start}};
+		}
 	}
+	else { $$meta{string} = join ' ', @words }
 
-	@words = $self->_do_aliases(@words) if @words && ! $$meta{broken};
-
-	return ($meta, @words);
+	$$meta{env}{ZOIDCMD} ||= $$meta{string}; # unix haters guide pdf page 60
+	return [$meta, @words];
 }
 
-sub _do_aliases {
+sub _do_aliases { # only a sub to be able to recurs
 	my ($self, $key, @rest) = @_;
 	if (exists $self->{aliases}{$key}) {
 		my $string = $self->{aliases}{$key}; # TODO Should we support other data types ?
@@ -338,28 +351,10 @@ sub _do_aliases {
 	else { return ($key, @rest) }
 }
 
-sub _no_words { # reconstruct original string
-	my ($self, $block) = @_;
-	my $string = $$block[0]{string};
-	bug 'need meta field "string" to do _no_words' unless $string;
-	if (exists $$block[0]{end}) {
-		$string =~ s/\s*\Q$_\E\s*$//
-			for reverse @{$$block[0]{end}};
-	}
-	if (exists $$block[0]{start}) {
-		$string =~ s/^\s*\Q$_\E\s*//
-			for @{$$block[0]{start}};
-	}
-	$$block[0]{_no_words}++;
-	$block = [ $$block[0], $string ];
-	return $block;
-}
-
-sub _do_words { # expand words etc.
+sub parse_words { # expand words etc.
 	my ($self, $block) = @_;
 #	@$block = $_->(@$block) for _stack($$self{contexts}, 'words_expansion');
 	@$block = $self->$_(@$block) for qw/_expand_param _expand_path/;
-	$self->{topic} = $$block[-1] unless $$block[0]{_no_topic};
 	return $block;
 }
 
@@ -367,7 +362,7 @@ sub _expand_param { # FIXME @_ implementation
 	my ($self, $meta, @words) = @_;
 	for (@words) {
 		next if /^'.*'$/;
-		s{ (?<!\\) \$ (?: \{ (.*?) \} | (\w+) ) (?: \[(\d+)\] )? }{
+		s{ (?<!\\) \$ (?: \{ (.*?) \} | (\w+) ) (?: \[(-?\d+)\] )? }{
 			my ($w, $i) = ($1 || $2, $3);
 			error "no advanced expansion for \$\{$w\}" if $w =~ /\W/;
 			$w = 	($w eq '_') ? $$self{topic} :
@@ -403,13 +398,16 @@ our $_NC_GLOB_OPTS = $_GLOB_OPTS | File::Glob::GLOB_NOCHECK();
 
 sub _expand_path { # path expansion
 	my ($self, $meta, @files) = @_;
-	unless ($self->{shell}{settings}{noglob}) {
-		@files = map { /^['"](.*)['"]$/ ? $1 : File::Glob::bsd_glob($_, 
-			$$self{settings}{allow_null_glob_expansion} ? $_GLOB_OPTS : $_NC_GLOB_OPTS
-		) } @files ;
-	}
-	else {  @files = map { /^['"](.*)['"]$/ ? $1 : $_ } @files  } # quote removal
-	return ($meta, @files);
+	return $meta, map { /^['"](.*)['"]$/ ? $1 : $_ } @files if $$self{settings}{noglob};
+	my $opts = $$self{settings}{allow_null_glob_expansion} ? $_GLOB_OPTS : $_NC_GLOB_OPTS;
+	return $meta, map {
+		if (/^['"](.*)['"]$/) { $1 }
+		else {
+			my @r = File::Glob::bsd_glob($_, $opts);
+			($_ !~ /^-/) ? (grep {$_ !~ /^-/} @r) : (@r);
+			# protect against implict switches as file names
+		}
+	} @files ;
 }
 
 # ############## #
@@ -443,6 +441,13 @@ sub _reload_file {
 
 sub dev_null {} # does absolutely nothing
 
+sub _stdin { # stub STDIN input
+	my (undef, $prompt) = @_;
+	local $/ = "\n";
+	print $prompt;
+	return <STDIN>;
+};
+
 # ########### #
 # Event logic #
 # ########### #
@@ -451,7 +456,7 @@ sub broadcast { # eval to be sure we return
 	my ($self, $event) = (shift(), shift());
 	return unless exists $self->{events}{$event};
 	debug "Broadcasting event: $event";
-	for my $sub (_stack($self->{events}, $event)) {
+	for my $sub (_stack($$self{events}, $event)) {
 		eval { $sub->($event, @_) };
 		complain("$sub died on event $event ($@)") if $@;
 	}
@@ -484,7 +489,7 @@ sub AUTOLOAD {
 		goto \&{$call};
 	}
 	else { # Shell like behaviour
-		debug 'No such object, trying to shell() it';
+		debug "No such method or object: '$call', trying to shell() it";
 		@_ = ([$call, @_]); # force words parsing
 		goto \&Zoidberg::Shell::shell;
 	}
@@ -496,95 +501,40 @@ sub AUTOLOAD {
 
 sub exit {
 	my $self = shift;
-	$self->Buffer->bell;
-	$self->{_continue} = 0;
-	# FIXME this should force the Buffer to quit
-	return 0;
+	if (@{$$self{jobs}} and ! $$self{_warned_bout_jobs}) {
+		complain "There are unfinished jobs";
+		$$self{_warned_bout_jobs}++;
+	}
+	else {
+		message join ' ', @_;
+		$self->{_continue} = 0;
+	}
+	# FIXME this should force ReadLine to quit
 }
 
 sub round_up {
 	my $self = shift;
 	$self->broadcast('exit');
 	if ($self->{round_up}) {
-		foreach my $n (keys %{$self->{objects}}) {
-			#print "debug roud up: ".ref( $self->{objects}{$n})."\n";
-			if (ref($self->{objects}{$n}) && $self->{objects}{$n}->isa('Zoidberg::Fish')) {
-				$self->{objects}{$n}->round_up;
-			}
-		}
+		tied( %{$$self{objects}} )->round_up(); # round up loaded plugins
 		Zoidberg::Contractor::round_up($self);
 
 		f_save_cache($self->{settings}{cache_dir}.'/zoid_path_cache');
 
 		undef $self->{round_up};
 	}
-	delete $Objects{"$self"};
-	return $self->{error} unless $self->{settings}{interactive}; # FIXME check this
+	return $$self{error} unless $$self{settings}{interactive};
 }
 
 sub DESTROY {
 	my $self = shift;
-	$self->round_up;
-	warn "Zoidberg was not properly cleaned up.\n"
-		if $self->{round_up};
+	if ($$self{round_up}) {
+		warn "Zoidberg was not properly cleaned up.\n";
+		$self->round_up;
+	}
+	delete $OBJECTS{"$self"};
 }
 
-# ############ #
-# Stub plugins #
-# ############ #
-
-package Zoidberg::stub;
-sub new { bless {'parent' => $_[1], 'config' => $_[2], 'zoidname' => $_[3]}, $_[0]; }
-sub help { return "This is a stub object -- it can't do anything."; }
-sub AUTOLOAD { return wantarray ? () : ''; }
-
-package Zoidberg::stub::prompt;
-use base 'Zoidberg::stub';
-sub stringify { return 'Zoidberg no prompt>'; }
-sub getLength { return length('Zoidberg no prompt>'); }
-
-package Zoidberg::stub::buffer;
-use base 'Zoidberg::stub';
-sub get_string {
-	my $self = shift;
-	$/ = "\n";
-	my $prompt = shift || "Zoidberg $Zoidberg::VERSION STDIN >>";
-	if (ref($prompt)) { $prompt = $prompt->stringify; }
-	print $prompt.' ';
-	if (defined (my $input = <STDIN>)) { return $input; } 
-	else { $self->{parent}->exit; }
-}
-sub size { return (undef, undef); } # width and heigth in chars
-sub bell { print "\007" }
-
-package Zoidberg::stub::history;
-use base 'Zoidberg::stub';
-sub get_hist { return (undef, '', 0); }
-
-package Zoidberg::stub::commands;
-use base 'Zoidberg::stub';
-use Zoidberg::Utils qw/:output/;
-sub parse {
-	if ($_[1] eq 'quit') { $_[0]->c_quit; }
-	return "";
-}
-sub list { return ['quit']; }
-sub c_quit {
-	my $self = shift;
-	if ($_[0]) { output $_[0] }
-	$self->{parent}->History->del_one_hist; # leave no trace
-	$self->{parent}->exit;
-}
-
-package Zoidberg::stub::intel;
-use base 'Zoidberg::stub';
-sub tab_exp { return [0, [$_[1]]]; }
-
-package Zoidberg::stub::help;
-use base 'Zoidberg::stub';
-sub help { print "No help available.\n" }
-sub list { return []; }
-
 1;
 
 __END__
@@ -595,62 +545,54 @@ Zoidberg - a modular perl shell
 
 =head1 SYNOPSIS
 
-Probably you just want the program F<zoid> which is the system command
-to start the Zoidberg shell. If you really want to initialize the module directly 
-you should check the code of F<zoid> for an elaborate example.
+You should use the B<zoid> system command to start the Zoidberg shell.
+If you want to initialize the module directly see the code of B<zoid> for an elaborate example.
 
 =head1 DESCRIPTION
 
-I<This page contains devel documentation, if you're looking for user documentation start with the zoid(1) man page>
+I<This page contains devel documentation, if you're looking for user documentation start with the zoid(1) man page.>
 
-This class provides an event and plugin engine.
+This class provides the main object of the Zoidberg shell, all other objects are nested 
+below attributes of this object.
+Also it contains some parser code along with methods to manage the events and plugin framework.
 
-FIXME more verbose description
+=head1 ATTRIBUTES
+
+FIXME - see also zoiddevel(1)
 
 =head1 METHODS
 
+FIXME list all methods
+
 Some methods:
 
 =over 4
 
-=item C<new()>
+=item C<new(%attr)>
 
-Simple constructor
-
-=item C<init(%attr)>
-
-Initialize secondary objects and sets config. C<%attr> is merged with C<%{$self}> and is ment for 
-runtime variables. System config should not be set here, but in L<Zoidberg::Config>.
+Initialize secondary objects and sets config. C<%attr> contains attributes to be used
+and is used to set runtime settings.
 
 =item C<main_loop()>
 
-Spans interactive shell reading from secondary object 'Buffer' or from STDIN. To quit this loop
-the routine C<exit()> of this package should be called.
+Spans interactive shell reading from a secondary ReadLine object or from STDIN.
+To quit this loop the routine C<exit()> of this package should be called.
 
 =item C<list_objects()>
 
-List secondary objects. These do not need to be loaded allready, the list is based on the config files.
-
-=item C<object($zoidname)>
-
-Returns secondary object stored under $zoidname. The first time this will "autoload" the object's packages and
-initialise it.
-
-=item C<test_object($zoidname, $class)>
-
-Returns true if object $zoidname exists and is blessed as $class. Since this calls C<object($zoidname)> the object
-will be initialised when tested.
+List secondary objects. These do not need to be loaded allready, 
+the list is based on the config files.
 
 =item C<exit()>
 
-Called by plugins to exit zoidberg -- this ends a interactive C<main_loop()> loop
+Called by plugins to exit zoidberg -- this ends a interactive C<main_loop()> loop.
+This does not clean up or destroy any objects, this means the C<main_loop()> can be 
+called again to restart it.
 
 =item C<round_up()>
 
-This method should be called B<before> letting the object being destroyed, it allows secundairy
-objects to be cleaned up nicely. If this is forgotten, C<DESTROY> will try to do this for you
-but an error will be printed to STDERR. From here all secundairy objects having
-a C<round_up()> method will be called recursively.
+This method should be called to clean up the shell objects.
+A C<round_up()> method will be called recursively for all secondairy objects.
 
 =back
 
@@ -666,7 +608,7 @@ modify it under the same terms as Perl itself.
 
 See L<http://www.perl.com/language/misc/Artistic.html>
 and L<http://www.gnu.org/copyleft/gpl.html>
-  
+
 =head1 SEE ALSO
 
 L<http://zoidberg.sourceforge.net>
